package work.xuye.common.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.enums.ResourceStatus;
import work.xuye.common.store.UrlMD5MapStore;

import java.util.Set;

/**
 * @author xuye
 * @since 2023/3/6 21:58
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class UrlMD5Service {

    private final UrlMD5MapStore urlMD5MapStore;

    public ResourceStatus getResourceStatus(String url, String md5) {
        String oldMd5 = urlMD5MapStore.get(url);
        if (oldMd5 == null) {
            return ResourceStatus.UNKNOWN;
        }
        if (oldMd5.equals(md5)) {
            return ResourceStatus.UNCHANGED;
        } else {
            return ResourceStatus.UPDATED;
        }
    }

    public void put(String url, String md5) {
        urlMD5MapStore.put(url, md5);
    }

    public void removeUrlMD5(String seedUrl) {
        Long aLong = urlMD5MapStore.removeKey(seedUrl);
        log.info("删除url对应的md5，url:{}, 删除条数:{}", seedUrl, aLong);
    }

    public void removeUrlCache(Message message) {
        Set<String> parentOtherCacheKeySet = message.getHeaders().get(MessageConstants.OTHER_CACHE_KEY_SET, Set.class);
        if (!ObjectUtils.isEmpty(parentOtherCacheKeySet)) {
            parentOtherCacheKeySet.forEach(this::removeUrlMD5);
        }
        String sendUrl = message.getHeaders().get(MessageConstants.SEED_URL, String.class);
        if (!StringUtils.hasText(sendUrl)) {
            this.removeUrlMD5(sendUrl);
        }
    }
}
